// Generated by Together

package com.itpath.framework.business;

import java.util.*;
import java.lang.reflect.*;
import java.text.SimpleDateFormat;

import com.itpath.framework.platform.interfaces.BusinessServices;
import com.itpath.framework.presentation.PresentationObject;
import com.itpath.framework.presentation.PresentationRules;
import com.itpath.framework.data.DataObject;

import com.itpath.framework.business.BusinessRules;
import com.itpath.framework.business.validation.*;

public class DefaultBusinessObject implements BusinessObject 
{
  private String _sActionName = null;
  protected Hashtable _hBusinessData = null;
  private BusinessServices _businessServices = null;
  private String _sActionRules = null;
  private Hashtable _hRulesTable = null;

  protected PresentationObject _presentationObject = null;
  
  private final String REQUIRED="Yes";

  // This constructor needed to extend this business object class
  // to handle the most generic cases
  public DefaultBusinessObject() { }

  // This business object constructor receives an actionName
  public DefaultBusinessObject(String sActionName)
  {
    // Call the setActionName method to setup the actionName
    setActionName( sActionName);
  }

  // The set actionName stores the actionName and
  // constructs the actionRules name
  protected void setActionName( String sActionName)
  {
    // Store the actionName class variable
    _sActionName = sActionName;

    // The action rules is the actionName with "Rules" appended to it.
    _sActionRules = _sActionName + BusinessRules.RULES_TAG;
  }

  public void init(Hashtable hFormData, BusinessServices businessServices)
      throws Exception
  {
    // set class variables
    _hBusinessData = hFormData;
    _businessServices = (BusinessServices) businessServices;

    // get a handle to the presentation object
    _presentationObject = businessServices.getPresentationObject( _sActionName );

    // Initialize the form for processing of data
    boolean isInitialLoadingOfForm = initializeForm( _businessServices, _hBusinessData);

    // If this is the first time we are loading this page (for a given cycle).
    if ( isInitialLoadingOfForm == true)
    {
      // exit this method because we have loaded the new page for the user
      return;
    }

    // check for valid user-entered data
    boolean isValid = isValid();

System.out.println( "isValid is "+ isValid);

    // if the data is valid
    if (isValid)
    {

    // TODO: Fix this stuff
    Hashtable hDataValues = _hBusinessData;
    
     // if not the last page
     if (! _presentationObject.isLastPage())
     {
        // show next page
        _presentationObject.showNextPage( hDataValues );
     }
     else // this is the last page
     {
       // execute the transaction
       performTransaction( _businessServices, hDataValues );
     }
    }
    else // Invalid data
    {
      // Instruct the Presentation Object to show the previous
      // data and errors.      
      this.showErrors( businessServices, _presentationObject, _hBusinessData);
    }
  }

  protected boolean initializeForm(BusinessServices businessServices, Hashtable formData)
    throws Exception
  {
    // Initialize the businessData class variable
    _hBusinessData = formData;

    // check the initial parameters
    if ( _presentationObject == null)
    {
      throw new Exception ("Invalid use of initialize form method."
        +" PresentationObj must be initialized prior to calling this method.");
    }
    
    // initialize the returnValue to false
    boolean returnValue = false;

    // Retrieve the current page from the formData
    String currentPage = (String) formData.get(BusinessRules.CURRENT_PAGE);

    // If current page is not defined in the hashtable
    if ( currentPage == null)
    {
      // This action has been initiated from the main menu

      // Initialize the values of Current Page and Action Name in the
      // Session Data
      currentPage = "-1";

      // Create a new hashtable for the resetData
      Hashtable resetData = new Hashtable();
      resetData.put(BusinessRules.CURRENT_PAGE, currentPage);
      resetData.put(BusinessRules.ACTION_NAME, _sActionName);

      // show the first page
      businessServices.getPresentationObject( _sActionName).showNextPage( resetData);

      returnValue = true;
    }
    else
    {
      // This action has been initiated from a prior page
      // i.e., page 2 of 4

      // call the setCurrentPage method to tell the presentation
      // object what page is being processed
      setCurrentPage( _presentationObject, formData);

      returnValue = false;
    }

    return returnValue;
  }
  
  protected String setCurrentPage( PresentationObject presentationObject, Hashtable formData)
  {
    // Set the current page in the presentation object
    String currentPage = (String) formData.get(BusinessRules.CURRENT_PAGE);

    presentationObject.setCurrentPage( currentPage);

    return currentPage;
  }

  // The showErrors method will display the error information to
  // the user.
  protected void showErrors( BusinessServices businessServices,
    PresentationObject presentationObject, Hashtable formData) throws Exception
  {
    // call the presentation object to display the errors contained in the
    // session data hashtable
    _presentationObject.showErrors( formData);
  }

  public boolean performTransaction(BusinessServices businessServices, Hashtable dataValues)
      throws Exception
  {
    // do transaction stuff
    DataObject dataObject = businessServices.getDataObject(_sActionName);

    // get the current date and format it according to the SQL Server format
    String datePattern = "MMM dd yyyy hh:mm:ss";
    SimpleDateFormat date = new SimpleDateFormat();
    date.applyPattern(datePattern);
    String formattedDate = date.format( new Date() );
    String lowerCased = formattedDate.toLowerCase();

    // add the date of the transaction to the session
    dataValues.put(BusinessRules.DATE_OF_TRANSACTION, lowerCased);

    // Build the sql string
    int result = dataObject.buildSQLString( dataValues);

    // Start the transaction
    dataObject.startTransaction();

    Vector vResults = new Vector();
    
    // Execute the transaction
    try
    {
      vResults = dataObject.execute();
    }
    catch ( Exception oce )
    {
      // we have a problem with connecting to Oracle, so clean up any left over
      // open transactions

      dataObject.rollbackTransaction();

      // re-throw the Exception
      throw oce;
    }

    // Commit the transaction
    result = dataObject.commitTransaction();

    // If the commit fails.
    if ( result != 0 )
    {
        throw new Exception("Oracle commit has failed");
    }

    return true;
  }

  // This method attempts to load the business rules corresponding
  // to this action.  These rules include the required status of the
  // fields, formatting definition, and grouping of data, if any.
  private Hashtable loadBusinessRules() throws Exception
  {
    // Initialize the rules table to null
    _hRulesTable = null;

    try {
      // Load the BusinessRules Class
      Class c = Class.forName( BusinessRules.BusinessRulesLocation + BusinessRules.BusinessRulesName);

      // Get the hashtable based upon the action name (using reflection)
      _hRulesTable = (Hashtable) c.getField(_sActionRules).get(null);

    } catch (Exception e) {

      // The hashtable was not found.  All actions will have some form of
      // rule interface.

      throw new Exception("No business rules exist for this Action," + _sActionRules );
    }

    return _hRulesTable;
  }

private void printHashtable ( Hashtable h)
{
  Enumeration e = h.keys();
  while ( e.hasMoreElements() )
  {
    String key = (String) e.nextElement();
    String value = (String) h.get(key);
    System.out.println(key +","+ value);
  }
}

  public boolean isValid() throws Exception
  {
    // initialize the return value
    boolean returnValue = true;

    // Initialize the field level validation
    boolean isFieldValid = true;

    // Load the rules table from the business rules object
    // this method may be moved to a utility class later!
    _hRulesTable = loadBusinessRules();

    // Retrieve the keys from the businessData hashtable
    Enumeration keys = _hBusinessData.keys();

    // While there are still keys remaining in the business data
    while ( keys.hasMoreElements() )
    {
      // Store the key name
      String fieldName = (String) keys.nextElement();

      // Store the user-entered value
      String userValue = (String) _hBusinessData.get( fieldName);

      // Retrieve the key value from the rules table
      BusinessRule rule = (BusinessRule) _hRulesTable.get(fieldName);

      // the field is in the mandatory hash table
      if ( rule != null)
      {
        // the field is mandatory, either because it is required
        // or because of a group dependency.

        // retrieve rules
        String fieldType = rule.getValidation();
        boolean isFieldRequired = rule.isRequired();

        /* Field-level Validation */

        // trim the extra spaces from the field
        userValue = userValue.trim();

        // check if the field is blank
        boolean fieldNotBlank = (userValue.length() > 0);

        if ( fieldNotBlank)
        {
          // if not blank, perform field-level validation
          isFieldValid = validateField( fieldName, fieldType, userValue);
        }
        else if ( isFieldRequired)
        {
          // if the field is blank, it may or may not be valid, depending on
          // whether or not it is required.

          // If the field is required, it is not valid
          // If the field is not required, it is valid
          isFieldValid = false;

        }

        // If the field is not valid, a missing tag will appear
        if ( !isFieldValid)
        {
          // NOT IMPLEMENTED
        }

      }
      else
      {
        // If this is not a field we care about
        if ( ignoreField( fieldName) )
        {
          // just ignore here
        }
        else
        {
          // the value is missing from the table
          // there's a BIG problem!!!

          throw new Exception( "missing key value for key " + fieldName + " in rules table" );
        }

      }

      // if there's an error, add the error to the businessData

      // The return value is the previous value AND the current field-level value
      returnValue = returnValue && isFieldValid;

      // reset the field-level validation for the next iteration
      isFieldValid = true;

    }  // end of field level validation

    if ( returnValue == false )
    {
      // NOT IMPLEMENTED
    }
    
    return returnValue;
  }

  private boolean ignoreField(String fieldName)
  {
    // If the field is not contained in the ValidationRules vector,
    // it is to be ignored
    boolean testOne = BusinessRules.ValidationRules.contains(fieldName);

    int index = fieldName.lastIndexOf("_");

    String fieldNameTemp = "";
    boolean testTwo = false;

    if ( index != -1)
    {
      fieldNameTemp = fieldName.substring(index);

      // If the field contains other tags that are to be ignored
      testTwo = BusinessRules.ValidationRules.contains(fieldNameTemp);
    }

    // If the field does not begin with Framework, it is to be ignored
    boolean testThree = !fieldName.startsWith(BusinessRules.Framework);

    // If both are true, the field is to be ignored
    return testOne || testTwo || testThree;
  }

  private boolean validateField( String fieldName, String fieldType, String fieldData )
    throws Exception
  {
    // trim all excess spaces around the user entered data
    fieldData = fieldData.trim();
    
    // load the field type validator
    String fieldNameClass = BusinessRules.BusinessRulesLocation + BusinessRules.Validation + fieldType + "Validator";
    ValidationObject validator = null;

    try
    {
      Class c = Class.forName( fieldNameClass );

      // Create the arguments contained in the constructor
      Class args[] = new Class[2];
      args[0] = fieldName.getClass();
      args[1] = fieldData.getClass();

      // get the constructor from the class
      Constructor con = c.getConstructor( args );

      // create the arguments for the constructor
      Object[] parameters = new Object[2];
      parameters[0] = fieldName;
      parameters[1] = fieldData;

      // create the constructor with the fieldData parameter
      validator = ( ValidationObject ) con.newInstance( parameters );

    }
    catch ( ClassNotFoundException e )
    {
      // Load the default validator
      validator = new DefaultFieldValidator( fieldName, fieldData);
    }
    catch ( Exception e3 )
    {
      // big problem... throw exception
      throw new Exception(
        "Attempting to load custom field validator for "+ fieldName +"\n" + e3.getMessage() );
    }


    // test the field level validator against the field data
    boolean isFieldValid = validator.isValid();

    if ( !isFieldValid)
    {
      generateErrorMessage( fieldName + BusinessRules.ERROR_TAG,
                            validator.getErrorMessage()    );
    }

    return isFieldValid;
  }

  public void generateErrorMessage( String field, String errorMessage)
  {
  }
}
