// Generated by Together

package com.itpath.framework.data;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;

import com.itpath.framework.business.BusinessRules;
import com.itpath.framework.platform.interfaces.DataServices;

public class DefaultDataObject implements DataObject 
{
  private DataServices _dataServices = null;
  private String _sActionName = null;

  private String _sSQLString = null;

  private Vector _vRulesVector = null;

  private Vector _vSQLStringVector = null;
  private Hashtable _hSQLData = null;

  public DefaultDataObject(String sActionName, DataServices dataServices)
  {
    // store the input parameters locally
    _sActionName = sActionName;
    _dataServices = dataServices;

    // initialize local variable(s)
    _vSQLStringVector = new Vector();

    // look up data rules object
    try
    {
      // Get the class.
      Class c = Class.forName( DataRules.className );
      String rulesClassName = _sActionName + "Rules";

      // retrieve vector for this action name
      _vRulesVector = (Vector) c.getField(rulesClassName).get(null);

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public int startTransaction() throws Exception
  {
    return _dataServices.startTransaction();
  }

  public int rollbackTransaction() throws Exception
  {
    return _dataServices.rollbackTransaction();
  }

  public int commitTransaction() throws Exception
  {
    return _dataServices.commitTransaction();
  }

  private String replaceTokens( String sqlString, Hashtable hFormData) throws Exception
  {
    String outputString = "";

    // Store the formData hashtable as a class variable
    _hSQLData = hFormData;

    StringTokenizer strTok = new StringTokenizer( sqlString, DataRules.BEGIN);
    String data = null;
    String field = null;
    String value = null;
    String insertString = null;

    // enumerate through fields
    while ( strTok.hasMoreElements() )
    {
      // get all characters until the next token.
      data = (String) strTok.nextElement();
    
      int location = data.indexOf( DataRules.END);
      if ( location  != -1)
      {
    
        // found the end delim

        // get the field from the string (sets isRepeatingStatement)
        field = getField( data, location);

        // if the hashtable does not contain the key
        //   and the field is not the sequence number
        if ( !hFormData.containsKey( field) )
        {

            // the field is missing from a required statement
            // This is a security exception.

            // Data elements are missing from a mandatory transaction.
            // This has occurred due to user attempts to deceive the system
            // by tampering with the forms.
            throw new Exception(
              "Data necessary to complete the transaction was not supplied."+
              "This field ["+ field +"] is missing." );
        }

        // get the value
        value = (String) _hSQLData.get(field);
        
        // Find the index of the underscore
        int index = value.indexOf("_");

        // If the value begins with FW and contains an underscore
        if (  value.startsWith(BusinessRules.Framework) && index != -1)
        {
          // Get the substring from the underscore
          value = value.substring(index + 1);
        }

        // The insert string is a combination of the value and the data.
        insertString = value + data.substring( location + 1);

      }
      else
      {
        // not found
        insertString = data;
      }

      // append to the class variable
      outputString += insertString;
    }

    // Initialize the returnString to null
    return outputString;
  }


  private String getField( String data, int location)
  {
    // get the field from the data element
    String field = data.substring(0, location);

    return field;
  }

  public int buildSQLString( Hashtable sessionData)
    throws Exception
  {
    // get an enumeration from the vector that contains the sql strings
    Enumeration enum = _vRulesVector.elements();

    // Initialize the template string
    String templateString = null;

    // Enumerate through the strings to build each sql statement
    while ( enum.hasMoreElements() )
    {
        // get the stored procedure string out from the vector
        templateString = (String) enum.nextElement();

        // Perform the actions to build the sql string at least once.
        // if it is repeating it will be repeated until complete.

        // replace the tokens in the templateString
        String completeSqlString = replaceTokens( templateString, sessionData);

        // if the returned string is not null (i.e., it is complete)
        if ( completeSqlString != null &&
             !completeSqlString.equals(DataRules.BLANK) )
        {
          // add the rowVector to the sqlStringVector.
          _vSQLStringVector.addElement(completeSqlString);
        }

    }

    return 0;
  }

  public Vector execute() throws Exception
  {
    // Get an enumeration of the sqlStringVector
    Enumeration enum = _vSQLStringVector.elements();

    // Instantiate the vReturnVector
    Vector vReturnVector = null;

    // If the enumeration is not null
    if ( enum != null )
    {
        // Enumerate through the elements of the vector
        while ( enum.hasMoreElements() )
        {
            // Get the sql and email strings from the vector
            String transactString = ( String ) enum.nextElement();

            // execute the query using the DataServices interface
            vReturnVector = _dataServices.execute( transactString );

            // If the results show error.
            if ( vReturnVector == null )
            {
                // Throw an oracle connectivity exception because there were problems
                // with the transaction
                throw new Exception( "problem storing data to Oracle!"
                  + "\nTransaction Info: " + transactString);
            }
        }
    }

    // Clear the sqlString vector
    _vSQLStringVector.removeAllElements();

    // return success
    return vReturnVector;
  }
}
