// Generated by Together

package com.itpath.framework.presentation;

import java.util.*;
import com.itpath.framework.platform.interfaces.PresentationServices;

public class DefaultPresentationObject implements PresentationObject 
{

  private java.util.Vector rulesVector = null;
  private PresentationServices _presentationServices;
  private int currPageInt = -1;

  // The constructor parameters include the actionName and the
  // PresentationServices interface reference.
  public DefaultPresentationObject(String actionName, PresentationServices presentationServices)
    throws Exception
  {
  System.out.println("ActionName in presentation object is "+ actionName);
  
    // Store a reference to the PresentationServices
    
    _presentationServices = presentationServices;

    // The rulesClassName is the actionName with the Rules tag afterwards
    String rulesClassName = actionName + PresentationRules.RULES;

    try {
      // Try to load the class for the PresentationRules
      Class c = Class.forName( PresentationRules.PresentationRulesLocation + PresentationRules.PresentationRulesName);

      // Attempt to retrieve the rules vector from the presentation object.
      rulesVector = (Vector) c.getField(rulesClassName).get(null);

    }
    catch (Exception e)
    {
      // Throw an exception indicating that something is missing from the
      // presentation rules.
      throw new Exception("The rules class does not contain a vector"
        +" for the attempted action.  Please check the PresentationRules for " + rulesClassName );
    }
  }

  public void setCurrentPage( String currPage)
  {
    try
    {
      // Parse the current page passed in from the HTML
      currPageInt = Integer.parseInt(currPage);
    }
    catch (Exception e)
    {
      // If the currentPage does not parse, set it to zero
      currPageInt = 0;
    }
  }

  // This method checks the current page against the rulesVector to
  // determine if we are through processing.  This is the only method with
  // knowledge of the number of forms in the transaction!
  public boolean isLastPage()
  {
    // the page numbers start at zero and the vector starts at one
    int rulesVectorAdjustedSize = rulesVector.size() - 1;

    // if the current page is less than the size of the rules vector
    if (currPageInt < rulesVectorAdjustedSize )
    {
      // this is not the last page, so return false
      return false;
    }

    // this is the last page, so return true
    return true;
  }

  // This method will show the next page in the sequence with
  // pre-filled data as included in the pageData Hashtable
  public void showNextPage(Hashtable pageData) throws Exception
  {
    // Retrieve the HTML page name from the rules vector for the next page
    String pageName = getHTMLPageName( ++currPageInt);

System.out.println( "pageName is "+ pageName);

    // call the PresentationServices interface method to evaluate the template
    // with the values filled in from the pageData Hashtable
    _presentationServices.evaluateTemplate( pageName, pageData );
  }

  // This method will show the current page with pre-filled data and
  // error messages as included in the errorData Hashtable
  public void showErrors(Hashtable errorData) throws Exception
  {
    // Retrieve the HTML page name from the rules vector for the current page
    String pageName = getHTMLPageName( currPageInt);

    // call the PresentationServices interface method to evaluate the template
    // with the values filled in from the pageData Hashtable
    _presentationServices.evaluateTemplate( pageName, errorData);
  }

  // This method will return the HTML filename corresponding to the pageNumber
  // passed in as a parameter
  private String getHTMLPageName(int pageNumber)
  {
    // return the HTML filename for the given pageNumber
    return (String) rulesVector.elementAt(pageNumber);
  }

  // This method will display a page that is independent of a sequence.
  // The pageName is contained in the pageData Hashtable
  public void showPage( Hashtable pageData) throws Exception
  {
    // Retrieve the pageName from the pageData hashtable
    String pageName = (String) pageData.get(PresentationRules.PAGE_NAME);

    // Call the evaluate template method of the PresentationServices object
    // to evaluate the pageName with the pageData hashtable values
    _presentationServices.evaluateTemplate(pageName, pageData);
  }

  /** This method will display an error form to the user with the capability
  * of inserting the errorMessage parameter as additional information
  * @deprecated This method is not currently used, but may be in the future.
  */
  public void showErrorPage(String errorMessage) throws Exception
  {
    // The pageName is stored in the PresentationRules
    String pageName = PresentationRules.ERROR_PAGE;

    // Create a new hashtable
    Hashtable pageData = new Hashtable();

    // Evaluate the template with the error message
    _presentationServices.evaluateTemplate(pageName, pageData);
  }
}