package com.itpath.security;

import java.sql.*;
import java.util.*;

public class JDBCHelper
{
	private static Connection connection = null;
	private static PreparedStatement _preparedStatementForUserQuery = null;
	private static String sUserQuery =
		"SELECT UserName From Users "+
		"WHERE ( ( Users.UserName = ? ) AND ( Users.Password = ? ) );";

	private static PreparedStatement _preparedStatementForFeatureRepositoryQuery = null;
	private static String sFeatureQuery =
		"SELECT PermissionName, PermissionURL From Permissions "+
		"WHERE ( Permissions.Application = ? );";

	private static PreparedStatement _preparedStatementForUserFeatureQuery = null;
	private static String sUserFeatureQuery =
		"SELECT Permissions.PermissionName, Permissions.PermissionURL "+
		"FROM Users, Permissions, User_Permission_Xref "+
		" WHERE ( "+
			"(User_Permission_Xref.PermissionID = Permissions.PermissionID) AND "+
			"(User_Permission_Xref.UserID = Users.UserID ) AND "+
			"(Permissions.Application = ?) AND "+
			"(Users.UserName = ? )"+
			" );";

	public static void initialize() throws SQLException
	{
		try
		{
			// open the jdbc connection to the database
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
			String url = "jdbc:odbc:example";
			String user = "";
			String pswd = "";

			connection = DriverManager.getConnection(url,user,pswd);

			// prepare a PreparedStatement for the query
			_preparedStatementForUserQuery = connection.prepareStatement( sUserQuery);
			_preparedStatementForFeatureRepositoryQuery = connection.prepareStatement(sFeatureQuery);
			_preparedStatementForUserFeatureQuery = connection.prepareStatement(sUserFeatureQuery);

		}
		catch (ClassNotFoundException cnfE)
		{
			throw new SQLException (cnfE.toString() );
		}
		catch (SQLException sqlE)
		{
			throw new SQLException (sqlE.toString() );
		}
	}

	public static boolean authenticateUser(String sApplicationName, String sUserName, String sPassword)
	{
		// for now, there's no difference
		return authenticateUser( sUserName, sPassword);
	}

	public static boolean authenticateUser(String sUserName, String sPassword)
	{
		boolean isAuthenticated = false;

		// ensure that the JDBC connection is still valid
		if ( connection == null)
		{
			return false;
		}

		try
		{
			// fill in the data values to the Prepared statement
			_preparedStatementForUserQuery.setString(1, sUserName);
			_preparedStatementForUserQuery.setString(2, sPassword);

			// execute the Query
			ResultSet resultSet = _preparedStatementForUserQuery.executeQuery();

			if ( !resultSet.next() )
			{
				// the database has no results - therefore the user is not authenticated
				return false;
			}

			// retrieve the resultset
			String sResult = resultSet.getString(1);

			// if the username is returned,
			if ( sResult != null)
			{
				// the user has been successfully authenticated
				isAuthenticated = sUserName.equals( sResult);
			}

		}
		catch (SQLException sqlE)
		{
			sqlE.printStackTrace();
			return false;
		}


		return isAuthenticated;
	}

	private static Hashtable getFeaturesFromResultSet( ResultSet resultSet) throws SQLException
	{

		Hashtable hFeatures = new Hashtable();

		String sFeatureName = null;
		String sFeaturePath = null;
		Feature feature = null;

		while ( resultSet.next() )
		{
			sFeatureName = resultSet.getString(1);
			sFeaturePath = resultSet.getString(2);

			feature = new FeatureImpl( sFeatureName, sFeaturePath, sFeatureName);

			hFeatures.put(sFeatureName, feature);
		}

		return hFeatures;
	}

	public static Hashtable loadFeatures(String sApplicationName)
	{
		Hashtable hFeatures = null;

		// ensure that the JDBC connection is still valid
		if ( connection == null)
		{
			return null;
		}

		try
		{
			// fill in the data values to the Prepared statement
			_preparedStatementForFeatureRepositoryQuery.setString(1, sApplicationName);

			// execute the Query
			ResultSet resultSet = _preparedStatementForFeatureRepositoryQuery.executeQuery();

			hFeatures = getFeaturesFromResultSet( resultSet);

		}
		catch (SQLException sqlE)
		{
			sqlE.printStackTrace();
			return null;
		}

		return hFeatures;
	}

	public static Hashtable loadFeaturesForUser(String sApplicationName, String sUserName)
	{
		Hashtable hFeatures = null;

		// ensure that the JDBC connection is still valid
		if ( connection == null)
		{
			return null;
		}

		try
		{
			// fill in the data values to the Prepared statement
			_preparedStatementForUserFeatureQuery.setString(1, sApplicationName);
			_preparedStatementForUserFeatureQuery.setString(2, sUserName);

			// execute the Query
			ResultSet resultSet = _preparedStatementForUserFeatureQuery.executeQuery();

			hFeatures = getFeaturesFromResultSet( resultSet);

		}
		catch (SQLException sqlE)
		{
			sqlE.printStackTrace();
			return null;
		}

		return hFeatures;
	}
}