package com.itpath.security;

import java.security.Principal;
import java.security.acl.*;
import sun.security.acl.*;
import java.util.*;

public final class WebSecurityManager
{
	private static final String _sSecurityOwner = "javaworld-example";
    private static final String _sGroupString = "javaworld-example";
	private static final String _sAclIdentifier = "javaworld-example";

	private static final Principal _securityOwner = new PrincipalImpl(_sSecurityOwner);
	private static final Group _securityGroup = new GroupImpl( _sGroupString);
	private static final Acl _aclExample = new AclImpl( _securityOwner, _sAclIdentifier);

	private static Hashtable _hFeatureList = null;

	public WebSecurityManager()
	{
	}

	public static void init(String sApplicationName) throws WebSecurityManagerException
	{
		try
		{
			// initialize the jdbc helper class
			JDBCHelper.initialize();

			// loads all the features associated with this application
			// into a hashtable
			_hFeatureList = JDBCHelper.loadFeatures(sApplicationName);
		}
		catch ( Exception e)
		{
			throw new WebSecurityManagerException("Error in init method", e);
		}
	}

	public static User retrieveUser(String sUserID, String sPassword, String sApplicationName)
		throws WebSecurityManagerException
	{
		// load user from database
		boolean userExists = JDBCHelper.authenticateUser(sApplicationName, sUserID, sPassword);

		// if not found, throw exception
		if ( !userExists)
		{
			throw new WebSecurityManagerException("User not found", new UserNotFoundException(sApplicationName, sUserID, "", null));
		}

		// if found, store the user into the session
		Hashtable features = loadFeaturesForUser(sApplicationName, sUserID);

		// Create the User object
		User user = new UserImpl(sUserID, new Hashtable() );

		// create the live acl entry for the user
		addAclEntry( user, features);

		return user;
	}

	private static Hashtable loadFeaturesForUser(String sApplication, String sUserID)
	{
		return JDBCHelper.loadFeaturesForUser(sApplication, sUserID);
	}

	public static final boolean isUserAuthenticated(User user, Feature feature)
	{
		// Just use the java.security.acl.ACL class to handle this
		return _aclExample.checkPermission(user, feature );
	}

	public static Feature getFeature(String sFeaturePath) throws WebSecurityManagerException
	{
		Feature feature = null;

		feature = (Feature) _hFeatureList.get( sFeaturePath);

		if ( feature == null)
		{
			throw new WebSecurityManagerException("The feature was not found",
												new FeatureNotFoundException(sFeaturePath) );
		}

		// return the feature stored in the hashtable of features
		return feature;
	}

	private static void addAclEntry(User user, Hashtable hFeatures) throws WebSecurityManagerException
	{
		// create a new Acl entry for this user
		AclEntry newAclEntry = new AclEntryImpl(  user);

		// initialize some temporary variables
		String sFeatureCode = null;
		Feature feature = null;

		Enumeration enumKeys = hFeatures.keys();
		String keyName = null;

		while ( enumKeys.hasMoreElements() )
		{
			// Get the key name from the enumeration
			keyName = (String) enumKeys.nextElement();

			// retrieve the feature from the hashtable
			feature = (Feature) hFeatures.get(keyName);

			// add the permission to the aclEntry
			newAclEntry.addPermission( feature );
		}

		try
		{
			// add the aclEntry to the ACL for the _securityOwner
			_aclExample.addEntry(_securityOwner, newAclEntry);
		} catch (NotOwnerException noE)
		{
			throw new WebSecurityManagerException("In addAclEntry", noE);
		}
	}
}